/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package FMG.IPcept;

import java.io.*;
import java.net.*;

/**
 * The ServerSocketControl class is intended to add functionality to the
 * Java ServerSocket class without writing a new socket implementation.
 * In particular, this class allows control of the IPcept proxy capability
 * on a bound socket.  This class requires natvie code support.
 * @author Mark Yarvis (yarvis@fmg.cs.ucla.edu)
 * @version 1.2
 */

public class ServerSocketControl {

   /**
    * Load the native code.
    */
   static {
      System.loadLibrary("IPceptCtl");
   }

   /**
    * Adds a port for which this bound socket will proxy.  This is 
    * essentially an interface to the IPCEPT_INTERCEPT setsockopt option.
    * This call can be made more than once to proxy for several ports.
    * @param s A previously created ServerSocket object that we want to use
    *          for proxying.
    * @param port The port number to proxy.  Connections directed to this
    *             port will be redirected to the indicated server socket.
    * @return No return value
    * @exception java.io.IOException is thrown if setsockopt returns an error.
    */
   public static native void addProxyPort(ServerSocket s, int port) 
                                                    throws IOException;
}
