/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <arpa/inet.h>

#include "util.h"
#include "../IPcept.h"

#include "FMG_IPcept_DatagramSocketControl.h"

JNIEXPORT void JNICALL Java_FMG_IPcept_DatagramSocketControl_addProxy
  (JNIEnv *env, jclass thisClass, jobject socket, jint port)
{
    int fd = getFDfromDatagramSocket(env, socket);
    int proxy_port = htons(port);
    int err = setsockopt(fd, SOL_IPCEPT, IPCEPT_INTERCEPT, &proxy_port,
        sizeof(int));

    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "setsockopt(IPCEPT_INTERCEPT) failed");
        return;
    }
    
    return;
}

JNIEXPORT void JNICALL Java_FMG_IPcept_DatagramSocketControl_removeProxy
  (JNIEnv *env, jclass thisClass, jobject socket, jint port)
{
    int fd = getFDfromDatagramSocket(env, socket);
    int proxy_port = htons(port);
    int err = setsockopt(fd, SOL_IPCEPT, IPCEPT_DONT_INTERCEPT, &proxy_port,
        sizeof(int));

    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "setsockopt(IPCEPT_INTERCEPT) failed");
        return;
    }
    
    return;
}


JNIEXPORT void JNICALL Java_FMG_IPcept_DatagramSocketControl_addMasqueradePair
  (JNIEnv *env, jclass thisClass, jobject socket, jobject destAddr, jint destPort,
   jobject srcAddr, jint srcPort)
{
    int fd = getFDfromDatagramSocket(env, socket);
    int err;
    struct sockaddr_in masq[2];

    jclass iaclass = env->FindClass("java/net/InetAddress");
    jfieldID addrID = env->GetFieldID(iaclass, "address", "I");

    masq[0].sin_family = AF_INET;
    masq[0].sin_addr.s_addr = htonl(env->GetIntField(destAddr, addrID));
    masq[0].sin_port = htons(destPort);

    masq[1].sin_family = AF_INET;
    masq[1].sin_addr.s_addr = htonl(env->GetIntField(srcAddr, addrID));
    masq[1].sin_port = htons(srcPort);

#if 0
    if (masq[1].sin_addr.s_addr == inet_addr("127.0.0.1")) {
        // Better Change to the real address of this host.
        struct sockaddr_in host_addr;
        struct hostent *hostptr;
        char buf[200];
        int len = 200;

        gethostname(buf, len);
        hostptr = gethostbyname(buf);
        host_addr.sin_family = AF_INET;
        host_addr.sin_addr.s_addr = ((struct in_addr*) hostptr->h_addr_list[0])->s_addr;
        
        masq[1].sin_addr.s_addr = host_addr.sin_addr.s_addr;
    }
#endif
    err = setsockopt(fd, SOL_IPCEPT, IPCEPT_MASQUERADE, masq, sizeof(masq));

    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "setsockopt(IPCEPT_MASQUERADE) failed");
        return;
    }
    
    return;
}

JNIEXPORT void JNICALL Java_FMG_IPcept_DatagramSocketControl_removeMasqueradeBySrc
  (JNIEnv *env, jclass thisClass, jobject socket, jobject jSrcAddr, jint jSrcPort)
{
    int fd = getFDfromDatagramSocket(env, socket);
    int err;
    struct sockaddr_in srcAddr;

    jclass iaclass = env->FindClass("java/net/InetAddress");
    jfieldID addrID = env->GetFieldID(iaclass, "address", "I");

    srcAddr.sin_family = AF_INET;
    srcAddr.sin_addr.s_addr = htonl(env->GetIntField(jSrcAddr, addrID));
    srcAddr.sin_port = htons(jSrcPort);

#if 0
    if (srcAddr.sin_addr.s_addr == inet_addr("127.0.0.1")) {
        // Better Change to the real address of this host.
        struct sockaddr_in host_addr;
        struct hostent *hostptr;
        char buf[200];
        int len = 200;

        gethostname(buf, len);
        hostptr = gethostbyname(buf);
        host_addr.sin_family = AF_INET;
        host_addr.sin_addr.s_addr = ((struct in_addr*) hostptr->h_addr_list[0])->s_addr;
        
        srcAddr.sin_addr.s_addr = host_addr.sin_addr.s_addr;
    }
#endif

    err = setsockopt(fd, SOL_IPCEPT, IPCEPT_DONT_MASQUERADE, &srcAddr, sizeof(srcAddr));

    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "setsockopt(IPCEPT_DONT_MASQUERADE) failed");
        return;
    }
    
    return;
}

JNIEXPORT jobject JNICALL Java_FMG_IPcept_DatagramSocketControl_getLastRecvDestAddr
  (JNIEnv *env, jclass thisClass, jobject socket)
{
    int fd = getFDfromDatagramSocket(env, socket);
    struct sockaddr_in sa;
    socklen_t len = sizeof(sa);
    int err;

    err = getsockopt(fd, SOL_IPCEPT, IPCEPT_LAST_RECV_DEST, &sa, &len);

    jclass iaclass = env->FindClass("java/net/InetAddress");
    jmethodID iainit = env->GetMethodID(iaclass, "<init>", "()V");
    jobject ia = env->NewObject(iaclass, iainit);

    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "getsockopt(IPCEPT_LAST_RECV_DEST) failed");
        return ia;
    }
    
    jfieldID addrID = env->GetFieldID(iaclass, "address", "I");
    env->SetIntField(ia, addrID, ntohl(sa.sin_addr.s_addr));
    
    return ia;
}

JNIEXPORT jint JNICALL Java_FMG_IPcept_DatagramSocketControl_getLastRecvDestPort
  (JNIEnv *env, jclass thisClass, jobject socket)
{
    int fd = getFDfromDatagramSocket(env, socket);
    struct sockaddr_in sa;
    socklen_t len = sizeof(sa);
    int err;

    err = getsockopt(fd, SOL_IPCEPT, IPCEPT_LAST_RECV_DEST, &sa, &len);
    
    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "getsockopt(IPCEPT_LAST_RECV_DEST) failed");
        return 0;
    }

    return ntohs(sa.sin_port);
}

JNIEXPORT void JNICALL Java_FMG_IPcept_DatagramSocketControl_setDontProxy
  (JNIEnv *env, jclass thisClass, jobject socket)
{
    int fd = getFDfromDatagramSocket(env, socket);
    int err = setsockopt(fd, SOL_IPCEPT, IPCEPT_DONT_PROXY, NULL, 0);
    
    if (err != 0) {
        jclass excl = env->FindClass("java/io/IOException");
        env->ThrowNew(excl, "setsockopt(IPCEPT_DONT_PROXY) failed");
        return;
    }

    return;
}

