/*
 * Copyright (c) 2001 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "util.h"

int getFDfromFileDes(JNIEnv *env, jobject fdes) {
    jclass fdesclass = env->GetObjectClass(fdes);
    jfieldID fdID = env->GetFieldID(fdesclass, "fd", "I");
#ifdef JDK1_2
    return env->GetIntField(fdes, fdID);
#else
    return env->GetIntField(fdes, fdID) - 1;
#endif
}

int getFDfromSocketImpl(JNIEnv *env, jobject impl) {
    jclass implclass = env->GetObjectClass(impl);
    jfieldID filedesID = env->GetFieldID(implclass, "fd", 
                                         "Ljava/io/FileDescriptor;");
    jobject fdesobj = env->GetObjectField(impl, filedesID);
    
    return getFDfromFileDes(env, fdesobj);
}

int getFDfromSocket(JNIEnv *env, jobject socket) {
    jclass sockclass = env->GetObjectClass(socket);
    jfieldID implID = env->GetFieldID(sockclass, "impl", 
                                      "Ljava/net/SocketImpl;");
    jobject impl = env->GetObjectField(socket, implID);
    
    return getFDfromSocketImpl(env, impl);
}


int getFDfromDatagramSocketImpl(JNIEnv *env, jobject impl)
{
    jclass implclass = env->GetObjectClass(impl);
    jfieldID filedesID = env->GetFieldID(implclass, "fd", 
        "Ljava/io/FileDescriptor;");
    jobject fdesobj = env->GetObjectField(impl, filedesID);
    
    return getFDfromFileDes(env, fdesobj);
}


int getFDfromDatagramSocket(JNIEnv *env, jobject socket)
{
    jclass sockclass = env->GetObjectClass(socket);
    jfieldID implID = env->GetFieldID(sockclass, "impl", 
        "Ljava/net/DatagramSocketImpl;");
    jobject impl = env->GetObjectField(socket, implID);
    
    return getFDfromSocketImpl(env, impl);
}
